@@ -16,43 +16,61 @@ DESC = """
   A configuration file (named 'config.json') is also required.        
   """
 
+# Weekday name to integer correspondence from GTFS
+INT_BY_WEEKDAY = {
+    'monday': 0,
+    'tuesday': 1,
+    'wednesday': 2,
+    'thursday': 3,
+    'friday': 4,
+    'saturday': 5,
+    'sunday': 6,
+}
+
 # Character to separate different chunks within an ID
 SEP = '#'
 
-def seconds_to_timestr(seconds, inverse=False):
+def timestr_to_seconds(x, inverse=False, mod24=False):
     """
-    Return the given number of integer seconds as the time string '%H:%M:%S'.
-    If ``inverse == True``, then do the inverse operation.
+    Given a time string of the form '%H:%M:%S', return the number of seconds
+    past midnight that it represents.
     In keeping with GTFS standards, the hours entry may be greater than 23.
+    If ``mod24 == True``, then return the number of seconds modulo ``24*3600``.
+    If ``inverse == True``, then do the inverse operation.
+    In this case, if ``mod24 == True`` also, then first take the number of 
+    seconds modulo ``24*3600``.
     """
     if not inverse:
         try:
-            seconds = int(seconds)
+            hours, mins, seconds = x.split(':')
+            result = int(hours)*3600 + int(mins)*60 + int(seconds)
+            if mod24:
+                result %= 24*3600
+        except:
+            result = None
+    else:
+        try:
+            seconds = int(x)
+            if mod24:
+                seconds %= 24*3600
             hours, remainder = divmod(seconds, 3600)
             mins, secs = divmod(remainder, 60)
             result = '{:02d}:{:02d}:{:02d}'.format(hours, mins, secs)
         except:
             result = None
-    else:
-        try:
-            hours, mins, seconds = seconds.split(':')
-            result = int(hours)*3600 + int(mins)*60 + int(seconds)
-        except:
-            result = None
     return result
 
-def timestr_mod_24(timestr):
+def get_bitlist(days_active):
     """
-    Given a GTFS time string in the format %H:%M:%S, return a timestring
-    in the same format but with the hours taken modulo 24.
+    Given a list of weekday names (e.g. ['sunday', 'tuesday']),
+    convert the weekday names to their corresponding integers 
+    ('monday'=0, 'tuesday'=1, ..., 'sunday'=6) to form a second list
+    ``s``, and return a list of seven bits, where a 1 in position ``i``
+    indicates that ``i`` lies in ``s`` and a 0 indicates otherwise.
+    Used to help create the GTFS file ``calendar.txt``.
     """
-    try:
-        hours, mins, seconds = [int(x) for x in timestr.split(':')]
-        hours %= 24
-        result = '{:02d}:{:02d}:{:02d}'.format(hours, mins, seconds)
-    except:
-        result = None
-    return result
+    days_active = {INT_BY_WEEKDAY[d] for d in days_active}
+    return [int(j in days_active) for j in range(7)]
 
 def get_duration(timestr1, timestr2, units='s'):
     """
@@ -65,8 +83,7 @@ def get_duration(timestr1, timestr2, units='s'):
     assert units in valid_units,\
       "Units must be one of {!s}".format(valid_units)
 
-    duration = seconds_to_timestr(timestr2, inverse=True) -\
-      seconds_to_timestr(timestr1, inverse=True)
+    duration = timestr_to_seconds(timestr2) - timestr_to_seconds(timestr1)
 
     if units == 's':
         return duration
@@ -76,10 +93,10 @@ def get_duration(timestr1, timestr2, units='s'):
         return duration/3600
 
 def get_shape_id(route_id):
-    return SEP.join(['s', route_id])
+    return SEP.join(['shp', route_id])
 
 def get_stop_ids(route_id):
-    return [SEP.join(['st', route_id, str(i)]) for i in range(2)]
+    return [SEP.join(['stp', route_id, str(i)]) for i in range(2)]
 
 def get_stop_names(route_short_name):
     return ['Route {!s} stop {!s}'.format(route_short_name, i)
@@ -100,8 +117,11 @@ def parse_args():
     parser.add_argument('input_dir', nargs='?', type=str, default='.',
       help='path to a directory containing the input files '\
       '(default: current directory)')
-    parser.add_argument('-o', dest='output_file',
-      help="path to the GTFS output file (default: 'gtfs.zip')")
+    parser.add_argument('-o', dest='output_dir', 
+      help="path to the output directory (default: input_dir)")
+    parser.add_argument('-z', dest='as_zip', action='store_true', 
+      default=False,
+      help='Write the output as a zip file instead of a collection of text files (default: False')
     return parser.parse_args()
 
 def main():
@@ -114,7 +134,7 @@ def main():
     # Create and export feed
     feed = Feed(args.input_dir)
     feed.create_all()
-    feed.export(args.output_file)
+    feed.export(args.output_dir, as_zip=args.as_zip)
 
 
 class Feed(object):
@@ -134,6 +154,7 @@ class Feed(object):
         # Import files
         self.config = json.load(open(
           os.path.join(input_dir, 'config.json'), 'r'))
+        self.service_windows = self.config['service_windows']
         self.raw_shapes = json.load(open(
           os.path.join(input_dir,'shapes.geojson'), 'r'))        
         raw_routes = pd.read_csv(
@@ -160,20 +181,6 @@ class Feed(object):
         # Save
         self.raw_routes = raw_routes
 
-    def get_service_windows(self):
-        """
-        Return ``self.config['service_windows']``.
-        """
-        return self.config['service_windows']
-
-    def get_window_duration(self, window_name, units='s'):
-        """
-        Return the duration of the given service window in the given units.
-        Allowable units are 's' (seconds), 'min' (minutes), 'h' (hours).
-        """
-        window = self.get_service_windows()[window_name]
-        return sum(get_duration(*w, units=units) for w in window)
-
     def create_agency(self):
         """
         Create a Pandas data frame representing ``agency.txt`` and save it to
@@ -189,21 +196,36 @@ class Feed(object):
         """
         Create a Pandas data frame representing ``calendar.txt`` and save it to
         ``self.calendar``.
-        It is a dumb calendar with one service that operates
-        on every day of the week.
+        Create the services from the distinct ``days_active`` fields of 
+        ``self.service_windows``.
+        Also create a dictionary ``self.service_by_swname`` with the structure
+        service window name -> service ID.
         """
-        self.calendar = pd.DataFrame({
-          'service_id': 'c0', 
-          'monday': 1,
-          'tuesday': 1, 
-          'wednesday': 1,
-          'thursday': 1, 
-          'friday': 1, 
-          'saturday': 1, 
-          'sunday': 1, 
-          'start_date': self.config['start_date'], 
-          'end_date': self.config['end_date'],
-          }, index=[0])
+        # Create a service ID for each distinct days_active field and map the
+        # service windows to those service IDs
+        def get_sid(bitlist):
+            return 'srv' + ''.join([str(b) for b in bitlist])
+
+        bitlists = set()
+        d = dict()
+        for sw in self.service_windows:
+            bitlist = get_bitlist(sw['days_active'])
+            bitlists.add(tuple(bitlist))
+            d[sw['name']] = get_sid(bitlist)
+
+        # Save d
+        self.service_by_swname = d
+
+        # Create calendar
+        start_date =  self.config['start_date']
+        end_date = self.config['end_date']
+        F = []
+        for bitlist in bitlists: 
+            F.append([get_sid(bitlist)] + list(bitlist) +\
+              [start_date, end_date])
+        self.calendar = pd.DataFrame(F, columns=[
+          'service_id', 'monday', 'tuesday', 'wednesday', 'thursday','friday',
+          'saturday', 'sunday', 'start_date', 'end_date'])
 
     def create_routes(self):
         """
@@ -230,6 +252,9 @@ class Feed(object):
         coordinates.
 
         Will create ``self.routes`` if it does not already exist.
+
+        The route IDs of routes without shapes (routes in ``routes.csv`` but 
+        not in ``shapes.geojson``) will not appear in the resulting dictionary.
         """
         if not hasattr(self, 'routes'):
             self.create_routes()
@@ -266,6 +291,8 @@ class Feed(object):
         linestring_by_route = self.get_linestring_by_route(use_utm=False)
         for index, row in self.routes.iterrows():
             route = row['route_id']
+            if route not in linestring_by_route:
+                continue
             linestring = linestring_by_route[route]
             shape = get_shape_id(route)
             rows = [[shape, i, lon, lat] 
@@ -321,32 +348,33 @@ class Feed(object):
             self.create_shapes()
 
         # Create trip IDs and directions
-        #self.add_num_trips_per_direction()
         F = []
-        windows = self.get_service_windows()
         for index, row in pd.merge(self.raw_routes, self.routes).iterrows():
             route = row['route_id']
             shape = get_shape_id(route)
-            for wname, window in windows.items():
-                headway = row[wname + '_headway']
-                for subwindow in window:
+            for sw in self.service_windows:
+                swname = sw['name']
+                headway = row[swname + '_headway']
+                service = self.service_by_swname[swname]
+                for subwindow in sw['subwindows']:
                     start = subwindow[0]
                     duration = get_duration(*subwindow, units='s') 
-                    num_trips_per_direction = int(duration/(headway*60))
+                    try:
+                        num_trips_per_direction = int(duration/(headway*60))
+                    except ValueError:
+                        num_trips_per_direction = 0
                     for direction in range(2):
                         for i in range(num_trips_per_direction):
                             F.append([
                               route, 
-                              SEP.join(['t', route, wname, start, 
+                              SEP.join(['t', route, swname, start, 
                               str(direction), str(i)]), 
                               direction,
                               shape,
+                              service,
                               ])
         f = pd.DataFrame(F, columns=['route_id', 'trip_id', 'direction_id', 
-          'shape_id'])
-
-        # Create service IDs
-        f['service_id'] = self.calendar['service_id'].iat[0]
+          'shape_id', 'service_id'])
 
         # Save
         self.trips = f
@@ -372,6 +400,8 @@ class Feed(object):
           self.routes[['route_id', 'route_short_name']])
         for route, group in pd.merge(f, self.trips).groupby(
           'route_id'):
+            if route not in linestring_by_route:
+                continue
             length = linestring_by_route[route].length/1000  # kilometers
             speed = group['speed'].iat[0]  # kph
             duration = int((length/speed)*3600)  # seconds
@@ -379,14 +409,13 @@ class Feed(object):
             sids_by_direction = {0: sids_tmp, 1: sids_tmp[::-1]}
             for index, row in group.iterrows():
                 trip = row['trip_id']
-                junk, route, wname, base_timestr, direction, i =\
+                junk, route, swname, base_timestr, direction, i =\
                   trip.split(SEP)
                 direction = int(direction)
                 i = int(i)
                 sids = sids_by_direction[direction]
-                base_time = seconds_to_timestr(base_timestr,
-                  inverse=True)
-                headway = row[wname + '_headway']
+                base_time = timestr_to_seconds(base_timestr)
+                headway = row[swname + '_headway']
                 start_time = base_time + headway*60*i
                 end_time = start_time + duration
                 # Get end time from route length
@@ -397,7 +426,7 @@ class Feed(object):
           'arrival_time', 'departure_time'])
         g[['arrival_time', 'departure_time']] =\
           g[['arrival_time', 'departure_time']].applymap(
-          lambda x: seconds_to_timestr(x))
+          lambda x: timestr_to_seconds(x, inverse=True))
         self.stop_times = g
 
     def create_all(self):
@@ -415,19 +444,19 @@ class Feed(object):
     def export(self, output_dir=None, as_zip=False):
         """
         Assuming all the necessary data frames have been created
-        (as in create_all()), export them to CSV files in the given output
+        (as in create_all()), export them to CSV files to the given output
         directory.
         If ``as_zip`` is True, then instead write the files to a 
         zip archive called ``gtfs.zip`` in the given output directory.
         If ``output_dir is None``, then write to ``self.input_dir``.
         """
-        names = ['agency', 'calendar', 'routes', 'stops', 'trips',
+        names = ['agency', 'calendar', 'routes', 'stops', 'shapes', 'trips',
           'stop_times']
         for name in names:
             assert hasattr(self, name),\
               "You must create {!s}".format(name)
         
-        # Write files to a temporary directory 
+        # Write files 
         if output_dir is None:
             output_dir = self.input_dir
         if not os.path.exists(output_dir):
@@ -436,7 +465,7 @@ class Feed(object):
             path = os.path.join(output_dir, name + '.txt')
             getattr(self, name).to_csv(path, index=False)
 
-        # If requested, zip and delete files 
+        # If requested, zip files and then delete files 
         if as_zip:
             # Create a temporary directory and move CSV files there
             tmp_dir = os.path.join(output_dir, 'hello-tmp-dir')
